﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Diagnostics;

namespace LogAnalysiser
{
    public partial class UpgradeFrm : Form
    {
        public UpgradeFrm()
        {
            InitializeComponent();
        }

        UpgradeFormApplication.UpgradeWebService.Service1SoapClient client;

        private void UpgradeFrm_Load(object sender, EventArgs e)
        {
            BeginCheckAsync();
        }

        public bool CheckVersionAndUpgrade()
        {
            try
            {
                if (client == null)
                {
                    client = new UpgradeFormApplication.UpgradeWebService.Service1SoapClient();
                }

                string upgradeVersion = client.GetUpgradeVersion();
                string upgradeFileName = client.GetUpgradeFileName();

                string currentVersion = CommonUntil.GetApplicationVersionFromExeFile();

                if (String.IsNullOrEmpty(upgradeVersion))
                {
                    lblStatus.Invoke(new Action(delegate
                    {
                        lblStatus.Text = "当前没有最新版本。";
                    }));
                    btnUpgrade.Invoke(new Action(delegate
                    {
                        btnUpgrade.Enabled = false;
                    }));
                    return false;
                }
                if (String.IsNullOrEmpty(currentVersion))
                {
                    return false;
                }

                if (currentVersion.Equals(upgradeVersion))
                {
                    lblStatus.Invoke(new Action(delegate
                    {
                        lblStatus.Text = "当前没有最新版本。";
                    }));
                    btnUpgrade.Invoke(new Action(delegate
                    {
                        btnUpgrade.Enabled = false;
                    }));
                    return false;
                }

                lblStatus.Invoke(new Action(delegate
                {
                    lblStatus.Text = "当前存在最新版本" + upgradeVersion + "，点击更新。。。";
                }));

                return true;
            }
            catch
            {
                lblStatus.Invoke(new Action(delegate{lblStatus.Text = "不能连接远程主机获取更新，请检查网络连接！";}));
                btnUpgrade.Enabled = false;
                return false;
            }
        }


        //由于操作webservice是一个耗时的操作，这里使用异步方式
        private void BeginCheckAsync()
        {
            Func<bool> func = new Func<bool>(CheckVersionAndUpgrade);
            IAsyncResult iar = func.BeginInvoke(new AsyncCallback(EndCheckAsync), func);
            lblStatus.Text = "正在检测更新，请稍候。。。";
        }

        private void EndCheckAsync(IAsyncResult iar)
        {
            Func<bool> func = (Func<bool>)iar.AsyncState;
            bool flag = func.EndInvoke(iar);
            //OnExistUpgradeVersionEventHandler(flag);  //将检测结果抛出
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
            this.Dispose();
           //Process.Start(Application.StartupPath + "\\LogAnalysiser.exe");
        }

        private void btnUpgrade_Click(object sender, EventArgs e)
        {
            try
            {
                if (client != null)
                {
                    client = new UpgradeFormApplication.UpgradeWebService.Service1SoapClient();
                }

                string url = client.GetUpgradeFileName();
                string fileName = Application.StartupPath + "\\LogAnalysiser.exe";

                BeginDownload(url, fileName, upgradePrograssBar, lblStatus);
            }
            catch
            {
                lblStatus.Invoke(new Action(delegate { lblStatus.Text = "不能连接远程主机获取更新，请检查网络连接！"; }));
                btnUpgrade.Enabled = false;
                return;
            }
        }

        private void BeginDownload(string url,string filename,ProgressBar progress,Label label)
        {
            Action<string, string, ProgressBar, Label> action = new Action<string, string, ProgressBar, Label>(DownLoadVersion);
            action.BeginInvoke(url, filename, progress, label, new AsyncCallback(EndDownload), action);
        }

        private void EndDownload(IAsyncResult iar)
        {
            Action<string, string, ProgressBar, Label> action = (Action<string, string, ProgressBar, Label>)iar.AsyncState;
            action.EndInvoke(iar);

            lblStatus.Invoke(new Action(delegate
            {
                lblStatus.Text = "更新完毕。";
            }));

            System.Threading.Thread.Sleep(1000);

            string fileName = Application.StartupPath + "\\LogAnalysiser.exe";
            Process.Start(fileName);

            Application.Exit();
        }

        private void DownLoadVersion(string url,string filename,ProgressBar progress,Label label)
        {
            int percent = 0;
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(url);
            request.ContentType = @"application/octet-stream";
            request.Credentials = CredentialCache.DefaultCredentials;

            HttpWebResponse response = (HttpWebResponse)request.GetResponse();

            long totalBytes = response.ContentLength;  //获取文件字节数

            progress.Invoke(new Action(delegate
            {
                progress.Maximum = (int)totalBytes; 
            }));

            Stream responseStream = response.GetResponseStream(); //保存到内存
            Stream fileStream = new FileStream(filename, FileMode.Create);

            long totalDownloadBytes = 0;
            byte[] bytes = new byte[1024];
            int paragraphByteSize = responseStream.Read(bytes, 0, (int)bytes.Length); //一次性读取1024个字节
            while (paragraphByteSize > 0)
            {
                totalDownloadBytes += paragraphByteSize;  //当前已经读取的字节数
                fileStream.Write(bytes, 0, paragraphByteSize); //写入到文件
                progress.Invoke(new Action(delegate
                {
                    progress.Value = (int)totalDownloadBytes;
                }));
                
                paragraphByteSize = responseStream.Read(bytes, 0, (int)bytes.Length); //继续读取下一段

                percent = (int)((float)totalDownloadBytes / (float)totalBytes * 100);  //进度百分比

                label.Invoke(new Action(delegate
                {
                    label.Text = "当前已经更新：" + percent.ToString() + "%";
                }));
            }
            responseStream.Close();
            fileStream.Close();
        }


    }
}
