﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Text.RegularExpressions;

namespace LogAnalysiser
{
    /// <summary>
    /// 扩展类，主要用于操作RichTextBox着色
    /// </summary>
    public static class RichTextBoxEx
    {
        /// <summary>
        /// 扩展方法，利用正则实现着色
        /// </summary>
        /// <param name="richTextBox">传入的RichTextBox控件</param>
        /// <param name="input">输入文本</param>
        /// <param name="section">待着色关键字</param>
        /// <param name="color">颜色</param>
        public static void SetColorBox(this RichTextBox richTextBox,string input, string section, Color color)
        {
            richTextBox.Text = input;
            richTextBox.Font = new Font(new FontFamily("Bookman Old Style"),10);
            input = input.Replace("\r\n", "~");  //注意，\r\n会被认为是4字节，其实他只是一个字节，所以替换为一个字符来表示
            //匹配关键字            
            string regexGrammer = section;
            MatchCollection mc = GetMatchedValue(regexGrammer, input);
            foreach (Match match in mc) 
            { 
                int length = match.Length; 
                int index = match.Index; 
                richTextBox.Select(index, length); 
                richTextBox.SelectionColor = color; 
            }
        }
        /// <summary>
        /// 得到匹配文本
        /// </summary>
        /// <param name="grammer">正则表达式</param>
        /// <param name="input">输入文本</param>
        /// <returns>返回MatchCollection类型</returns>
        private static MatchCollection GetMatchedValue(string grammer, string input) 
        { 
            Regex digitRegex = new Regex(grammer, RegexOptions.IgnoreCase | RegexOptions.Singleline); 
            MatchCollection matchCollection = digitRegex.Matches(input);
            return matchCollection; 
        }
    }
}
