﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Configuration;
using System.Linq;
using System.Diagnostics;
using LogAnalysiser.FormRes;

namespace LogAnalysiser
{
    /// <summary>
    /// 主窗体，软件显示的主界面
    /// </summary>
    public partial class MainFrm : Form
    {
        #region Constructor
        /// <summary>
        /// 构造函数，初始化
        /// </summary>
        public MainFrm()
        {
            InitializeComponent();  
            mainFrm = this;
            tTick.Interval = 1000;
            tTick.Enabled = true;
            btnAnalysis.Enabled = false;

            if (operation == null)
            {
                operation = new XmlOperation();
            }
        }
        #endregion

        #region Public init variable
        /// <summary>
        /// 记录条数
        /// </summary>
        public static int filterCount = 0;
        /// <summary>
        /// 计时开关
        /// </summary>
        public static bool flag = false;  

        //private const UInt32 WM_ACTIVATE = 0x0006;
        /// <summary>
        /// 过滤开关
        /// </summary>
        public static bool noFilterRowFlag = true; 

        /// <summary>
        /// 配置窗体
        /// </summary>
        public ConfigFrm configFrm; 
        /// <summary>
        /// 着色窗体
        /// </summary>
        public DaemonFrm daemonFrm;
        /// <summary>
        /// 启动窗体
        /// </summary>
        public SlashFrm slashFrm;
        /// <summary>
        /// 主窗体
        /// </summary>
        public MainFrm mainFrm;
        /// <summary>
        /// 关于软件窗体
        /// </summary>
        public AboutFrm aboutFrm;
        /// <summary>
        /// 读取配置文件类
        /// </summary>
        public AppSettingsReader configReader ; 
        /// <summary>
        /// 操作XML类
        /// </summary>
        public XmlOperation operation;
        #endregion

        #region Append text in async mode
        /// <summary>
        /// 处理过滤数据
        /// </summary>
        private void LoadAppendingText()
        {
            try
            {
                string fileName = txtFileName.Text;
                StreamWriter writer = new StreamWriter(Application.StartupPath + "\\" + Path.GetFileNameWithoutExtension(fileName));

                using (StreamReader reader = new StreamReader(fileName))
                {
                    string logLine = string.Empty;


                    string value = operation.Read();

                     var filterWords = from p in value.Split('|').AsEnumerable()
                                      select p;

                    while ((logLine = reader.ReadLine()) != null)
                    {
                        filterCount++;
                        noFilterRowFlag = true;  //重新置位
                        foreach (string filterWord in filterWords)
                        {
                            noFilterRowFlag = noFilterRowFlag &&(!logLine.Contains(filterWord)); //是不是被过滤
                        }

                        if (noFilterRowFlag)
                        {
                            richTextBox1.Invoke(new Action(delegate
                            {
                                richTextBox1.AppendText(logLine + "\r\n");
                            }));

                            writer.WriteLine(logLine); //写文件
                            writer.Flush();
                        }
                        this.statusNotify.Invoke(new Action(delegate
                        {
                            pPrograss.Value = filterCount;
                            currentRecords.Text = filterCount.ToString();
                            //this.richTextBox1.ScrollToCaret();
                        }));

                    }
                    writer.Close();
                }
            }
            catch (System.IO.IOException ioEx)
            {
                MessageBox.Show(ioEx.Message, "Caution", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message, "Caution", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
        #endregion

        #region Init
        /// <summary>
        /// 窗体初始
        /// </summary>
        private void InitControlState()
        {
            if (String.IsNullOrEmpty(txtFileName.Text))
            {
                MessageBox.Show("Pls Select a file!!!");
                return;
            }
            if (!String.IsNullOrEmpty(richTextBox1.Text))
            {
                richTextBox1.Text = string.Empty;
            }

            if (pPrograss.Value != 0)
            {
                pPrograss.Value = 0;
            }

            if (totalRecords.Text != string.Empty)
            {
                totalRecords.Text = string.Empty;
            }

            if (currentRecords.Text != string.Empty)
            {
                currentRecords.Text = string.Empty;
            }

            filterCount = 0;

            tTick.Interval = 100;

        }
        #endregion

        #region Begin and End Invoke of Async mode
        /// <summary>
        /// 异步开始
        /// </summary>
        private void BeginInvokeAppending()
        {
            Action action = new Action(LoadAppendingText);
            IAsyncResult result = action.BeginInvoke(new AsyncCallback(EndInvokeAppending),action);
            pPrograss.Maximum = GetTotalCounts();
            tTick.Enabled = true;
        }

        /// <summary>
        /// 异步结束
        /// </summary>
        /// <param name="iar"></param>
        private void EndInvokeAppending(IAsyncResult iar)
        {
            btnAnalysis.Invoke(new Action(delegate 
                {
                    btnAnalysis.Enabled = false;
                }));
            tTick.Enabled = false;
            notificationIcon.Image = (Image)WinRes.Complete;
            Action action = (Action)iar.AsyncState;
            action.EndInvoke(iar);
        }
        #endregion

        #region Get Total Record counts
        /// <summary>
        /// 得到记录的总条数
        /// </summary>
        /// <returns></returns>
        private int GetTotalCounts()
        {
            if (String.IsNullOrEmpty(txtFileName.Text))
            {
                MessageBox.Show("Pls Select a file!!!");
                return 0;
            }
            string fileName = txtFileName.Text;
            int totalCount = 0;
            using (StreamReader reader = new StreamReader(fileName))
            {
                while (reader.ReadLine() != null)
                {
                    totalCount++;
                }
            }
            return totalCount;
        }
        #endregion

        #region Event
        /// <summary>
        /// 点击浏览按钮，用以选取待操作的文件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBrowser_Click(object sender, EventArgs e)
        {
            OpenFileDialog openedFile = new OpenFileDialog();
            openedFile.Filter = "All files(*.*)|*.*";

            if (openedFile.ShowDialog() == DialogResult.OK)
            {
                this.txtFileName.Text = openedFile.FileName;
                btnAnalysis.Enabled = true;
                richTextBox1.Text = string.Empty;
            }
        }

        /// <summary>
        /// 点击分析按钮，用于过滤符合条件的行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAnalysis_Click(object sender, EventArgs e)
        {
            InitControlState();

            totalRecords.Text = GetTotalCounts().ToString();

            BeginInvokeAppending();
        }

        /// <summary>
        /// 用以控制RichTextBox的滚动与否
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopScrollControl_Click(object sender, EventArgs e)
        {
            stopScrollControl.Text = ((stopScrollControl.Text == "StopScrolling") ? "StartScrolling" : "StopScrolling");
        }

        /// <summary>
        /// 时钟跳跃
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tTick_Tick(object sender, EventArgs e)
        {
            flag = !flag;
            if (flag == true)
            {
                notificationIcon.Image = (Image)WinRes.Normal;
            }
            else
            {
                notificationIcon.Image = (Image)WinRes.NormalNo;
            }
        }

        /// <summary>
        /// 点击配置按钮，可以输入待过滤的行所包含的关键字
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsBtnConfig_Click(object sender, EventArgs e)
        {
            if (configFrm == null || configFrm.IsDisposed == true )
            {
                configFrm = new ConfigFrm();
            }
            configFrm.Show();

        }

        /// <summary>
        /// 点击着色按钮，可以对当前文档进行关键字高亮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsBtnColor_Click(object sender, EventArgs e)
        {
            if (daemonFrm == null || daemonFrm.IsDisposed == true)
            {
                daemonFrm = new DaemonFrm();
            }
            daemonFrm.OnColorDaemonEventHandler += new ColorDaemonDelegate(daemonFrm_OnColorDaemonEventHandler);
            daemonFrm.Show();
        }

        /// <summary>
        /// 着色委托事件，可以实时高亮关键字
        /// </summary>
        /// <param name="text"></param>
        private void daemonFrm_OnColorDaemonEventHandler(string text)
        {
            RichTextBoxEx.SetColorBox(richTextBox1,richTextBox1.Text, text, Color.Red);
        }

        /// <summary>
        /// 打开帮助文档
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsBtnHelp_Click(object sender, EventArgs e)
        {
            string fileName = Application.StartupPath + "\\MyHelp.chm";
            if (!File.Exists(fileName))
            {
                MessageBox.Show("帮助文档不存在！");
                return;
            }
            Process.Start(fileName);
        }

        /// <summary>
        /// 关于本软件的介绍
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsBtnAbout_Click(object sender, EventArgs e)
        {
            if (aboutFrm == null || aboutFrm.IsDisposed == true)
            {
                aboutFrm = new AboutFrm();
            }
            aboutFrm.Show();
        }
       
        /// <summary>
        /// 主窗体加载事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainFrm_Load(object sender, EventArgs e)
        {
            this.Opacity = 0;
            this.ShowInTaskbar = false;
            if (slashFrm == null || slashFrm.IsDisposed == true)
            {
                slashFrm = new SlashFrm();
            }
            slashFrm.OnHookSlashCloseEvent+=new HookSlashCloseDelegate(slashFrm_OnHookSlashCloseEvent);
            slashFrm.Show();

            configReader = new AppSettingsReader();
        }

        /// <summary>
        ///SlashFrm窗体加载结束，显示主窗体。 
        /// </summary>
        private void slashFrm_OnHookSlashCloseEvent()
        {
            this.Opacity = 1;
            this.ShowInTaskbar = true;
        }

        /// <summary>
        /// 点击更新按钮，进行软件更新操作。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsUpdate_Click(object sender, EventArgs e)
        {
            string upgradeAssembleExePath = Application.StartupPath + "\\upgrade.exe";
            if (!File.Exists(upgradeAssembleExePath))
            {
                MessageBox.Show("更新程序不存在，更新失败！");
                return;
            }

            Process.Start(upgradeAssembleExePath);

            Application.Exit();
        }

        #endregion

       
    }
}
